@@ -1,5 +1,5 @@
 /*  Zutils - Utilities dealing with compressed files
-    Copyright (C) 2009-2014 Antonio Diaz Diaz.
+    Copyright (C) 2009-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,15 @@ const char * const simple_extensions[num_formats] =
 const int format_order[num_formats] =
   { fmt_lz, fmt_bz2, fmt_gz, fmt_xz };		// search order
 
+bool enabled_format( const int format_index );
+void parse_format_list( const std::string & arg );
+int parse_format_type( const std::string & arg );
+
+int extension_index( const std::string & name );	// -1 if unknown
+int extension_format( const int eindex );		// -1 if uncompressed
+const char * extension_from( const int eindex );
+const char * extension_to( const int eindex );
+
 extern const char * invocation_name;
 extern const char * program_name;
 extern int verbosity;
