@@ -1,5 +1,5 @@
 /*  Zutils - Utilities dealing with compressed files
-    Copyright (C) 2009-2014 Antonio Diaz Diaz.
+    Copyright (C) 2009-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -75,16 +75,6 @@ int child_status( const pid_t pid, const char * const name )
 } // end namespace
 
 
-int parse_format_type( const std::string & arg )
-  {
-  for( int i = 0; i < num_formats; ++i )
-    if( arg == format_names[i] )
-      return i;
-  show_error( "Bad argument for '--format' option." );
-  std::exit( 1 );
-  }
-
-
 // Returns the number of bytes really read.
 // If (returned value < size) and (errno == 0), means EOF was reached.
 //
